// cfg2000fr_evenements_v2.js
// Tout ce qui suit un double slash sont des commentaires, jusqu'à la fin de la ligne.
/*
Les blocs de lignes commençant par 'slash étoile' et terminant par 'étoile slash' sont des commentaires

Ceci est un fichier de configuration, qui permet de définir :
° la toolbox = l'ensemble des blocs disponibles
° la startblocs = les blocs présents au départ
° la définition des cases
° la définition des cartes présentes
° les fenêtres visibles et cachées
° la position des fenêtres
° la taille des fenêtres
° divers autres paramètres.

Ce fichier de configuration est un programme écrit en javascript.
Ceci rend la configuration très souple et puissante.
Il n'est pas nécessaire de tout comprendre pour prendre un fichier de configuration existant, 
faire quelques modification et obtenir ainsi un nouveau fichier de fonfiguration.

Voir le fichier : bglibrary_frame_card.js  pour des paramètres.  En particulier :
° gloCanvas, qui est de type : OneCanvas()
° glaoFrames, qui est un tableau d'éléments de type : OneFrame()
° Le nombre de cases (cadres, Frames) va de 0 à glnFramesMax
  La case n° 0 est en dehors de la fenêtre. Elle sert à faire disparaître ou apparaître des cartes.
° glaoTexts, qui est un tableau d'éléments de type : OneTextMemory()
  Ses indices vont de 0 à 9 (MAX_TEXT_MEMORY-1).
  Il sert à afficher du texte dans le jeu de cartes.
 */
//-------------------------------------------------
/*
Exemple de gestion d'événements
° Une ToolBox avec tous les blocs disponibles, les blocs de programme disponibles.
° Un exemple de StartBlocks, les blocs présents au départ.
° Les cartes bougent dans tous les sens,
  il faut cliquer sur l'As de pique pour les arrêter.
° Définition de paramètres.
° Définition des fenêtres visibles et cachées, de leurs tailles et positions.
 */
// ########################################################################################

function confToolBoxInit() {
//==========================
// Définition de la toolbox utilisée.
// La moindre erreur fait que la nouvelle toolbox est ignorée.
// Tous les blocs possibles sont disponibles ici.
// Pour un autre fichier de configuration, on peut tout copier,
// puis effacer les blocs non désirés.

// La toolbox sera constituée de différentes blocs.
var strToolbox = 
'<xml>\n'
+'\n  <category name="%{BKY_CATEGORY_NAME_LOGIC}" colour="%{BKY_LOGIC_HUE}">'
+'\n    <block type="controls_if"></block>'
+'\n    <block type="logic_compare"></block>'
+'\n    <block type="logic_operation"></block>'
+'\n    <block type="logic_negate"></block>'
+'\n    <block type="logic_boolean"></block>'
+'\n    <block type="logic_null"></block>'
+'\n    <block type="logic_ternary"></block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_LOOPS}" colour="%{BKY_LOOPS_HUE}">'
+'\n    <block type="controls_repeat_ext">'
+'\n      <value name="TIMES">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">10</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="controls_whileUntil"></block>'
+'\n    <block type="controls_for">'
+'\n      <value name="FROM">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="TO">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">10</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="BY">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="controls_forEach"></block>'
+'\n    <block type="controls_flow_statements"></block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_MATH}" colour="%{BKY_MATH_HUE}">'
+'\n    <block type="math_number">'
+'\n      <field name="NUM">123</field>'
+'\n    </block>'
+'\n    <block type="math_arithmetic">'
+'\n      <value name="A">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="B">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="math_single">'
+'\n      <value name="NUM">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">9</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="math_trig">'
+'\n      <value name="NUM">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">45</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="math_constant"></block>'
+'\n    <block type="math_number_property">'
+'\n      <value name="NUMBER_TO_CHECK">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">0</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="math_round">'
+'\n      <value name="NUM">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">3.1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="math_on_list"></block>'
+'\n    <block type="math_modulo">'
+'\n      <value name="DIVIDEND">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">64</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="DIVISOR">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">10</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="math_constrain">'
+'\n      <value name="VALUE">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">50</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="LOW">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="HIGH">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">100</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="math_random_int">'
+'\n      <value name="FROM">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="TO">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">100</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="math_random_float"></block>'
+'\n    <block type="math_atan2">'
+'\n      <value name="X">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="Y">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_TEXT}" colour="%{BKY_TEXTS_HUE}">'
+'\n    <block type="text"></block>'
+'\n    <block type="text_join"></block>'
+'\n    <block type="text_afficheln">'
+'\n      <value name="TEXT">'
+'\n        <shadow type="text"></shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_affiche">'
+'\n      <value name="TEXT">'
+'\n        <shadow type="text"></shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_length">'
+'\n      <value name="VALUE">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">abc</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_isEmpty">'
+'\n      <value name="VALUE">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT"></field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_indexOf">'
+'\n      <value name="VALUE">'
+'\n        <block type="variables_get">'
+'\n          <field name="VAR">{textVariable}</field>'
+'\n        </block>'
+'\n      </value>'
+'\n      <value name="FIND">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">abc</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_charAt">'
+'\n      <value name="VALUE">'
+'\n        <block type="variables_get">'
+'\n          <field name="VAR">{textVariable}</field>'
+'\n        </block>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_getSubstring">'
+'\n      <value name="STRING">'
+'\n        <block type="variables_get">'
+'\n          <field name="VAR">{textVariable}</field>'
+'\n        </block>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_changeCase">'
+'\n      <value name="TEXT">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">abc</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_trim">'
+'\n      <value name="TEXT">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">abc</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_prompt_ext">'
+'\n      <value name="TEXT">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">abc</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_TEXT} +" colour="%{BKY_TEXTS_HUE}">'
+'\n    <block type="text_comment">'
+'\n      <value name="COMMENT">'
+'\n        <shadow type="text">'
+'\n          <field id="XML_TB_COMMENT" name="TEXT">écrivez un commentaire...</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_affiche_efface">'
+'\n    </block>'
+'\n    <block type="variable_spy">'
+'\n      <value name="TIME">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1.5</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_pause">'
+'\n      <value name="TIME">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1.5</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="text_parse_to_number">'
+'\n      <value name="STR">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">123</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_CARD}" colour="%{BKY_COLOUR_HUE}">'
+'\n    <block type="card_get_card">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_flip">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <data>1</data>'
+'\n    </block>'
+'\n    <block type="card_movef1f2">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">3</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="FRAME2">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">5</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <data>1</data>'
+'\n    </block>'
+'\n    <block type="card_exchangef1f2">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">3</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="FRAME2">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">5</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <data>1</data>'
+'\n    </block>'
+'\n    <block type="card_set_shift">'
+'\n      <value name="SHIFT">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">10</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_text_write">'
+'\n        <value name="THETEXT">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT"></field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="POSX">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">150</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="POSY">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">100</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="THENUMBER">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">0</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_text_font_size_color">'
+'\n      <value name="THESIZE">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">20</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="THENUMBER">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">0</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_CARD} +" colour="%{BKY_COLOUR_HUE}">'
+'\n    <block type="card_add_one_card">'
+'\n      <value name="CARD">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">101</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="BACK">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">900</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="FRAME2">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <data>1</data>'
+'\n    </block>'
+'\n    <block type="card_remove_one_card">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <data>1</data>'
+'\n    </block>'
+'\n    <block type="card_remove_all">'
+'\n      <value name="ACCELERATE">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">6</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_exist_test">'
+'\n      <value name="CARD">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_exist_response">'
+'\n    </block>'
+'\n    <block type="card_source_define">'
+'\n      <value name="SOURCE">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">../images/Carte</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="EXT">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">.png</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_back_define">'
+'\n      <value name="BACK">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">900</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_CARD} ++" colour="%{BKY_COLOUR_HUE}">'
+'\n    <block type="card_link_func_card">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="FUNC">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT"></field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_get_card_param">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_set_card_param">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n      <value name="PARAM1">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT"></field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_get_card_click_count">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="card_reset_click_count">'
+'\n      <value name="FRAME1">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">1</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_LISTS}" colour="%{BKY_LISTS_HUE}">'
+'\n    <block type="lists_create_with">'
+'\n      <mutation items="0"></mutation>'
+'\n    </block>'
+'\n    <block type="lists_create_with"></block>'
+'\n    <block type="lists_repeat">'
+'\n      <value name="NUM">'
+'\n        <shadow type="math_number">'
+'\n          <field name="NUM">5</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="lists_length"></block>'
+'\n    <block type="lists_isEmpty"></block>'
+'\n    <block type="lists_indexOf">'
+'\n      <value name="VALUE">'
+'\n        <block type="variables_get">'
+'\n          <field name="VAR">{listVariable}</field>'
+'\n        </block>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="lists_getIndex">'
+'\n      <value name="VALUE">'
+'\n        <block type="variables_get">'
+'\n          <field name="VAR">{lCOMMENTistVariable}</field>'
+'\n        </block>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="lists_setIndex">'
+'\n      <value name="LIST">'
+'\n        <block type="variables_get">'
+'\n          <field name="VAR">{listVariable}</field>'
+'\n        </block>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="lists_getSublist">'
+'\n      <value name="LIST">'
+'\n        <block type="variables_get">'
+'\n          <field name="VAR">{listVariable}</field>'
+'\n        </block>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="lists_split">'
+'\n      <value name="DELIM">'
+'\n        <shadow type="text">'
+'\n          <field name="TEXT">,</field>'
+'\n        </shadow>'
+'\n      </value>'
+'\n    </block>'
+'\n    <block type="lists_sort"></block>'
+'\n  </category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_VARIABLES}" colour="%{BKY_VARIABLES_HUE}" custom="VARIABLE"></category>'
+'\n  <category name="%{BKY_CATEGORY_NAME_FUNCTIONS}" colour="%{BKY_PROCEDURES_HUE}" custom="PROCEDURE"></category>'
+'\n</xml>'
+'\n';

// Change le contenu de la toolbox, pour donner la possibilité d'avoir une fonction.
gloB.demoWorkspace.updateToolbox(strToolbox);
} // confToolBoxInit

function confStartBlocksInit() {
//==============================
// Définition des blocs présents au départ
// C'est long mais simple à faire.
// On :
// ° place les blocs désirés dans ex2000.html
// ° Fichiers enregistrer sous ...
// ° demande de voir le fichier dans un éditeur de texte.
// ° fait un copier-coller du texte.
// ° ajoute des  +'\n'  en début de chaque ligne (sauf la première)
// ° ajoute des  '  en fin de ligne.

//return; // Ne fait rien, pour des tests.

gloB.demoWorkspace.clear();  // Enlève tous les blocs, c.f. : https://developers.google.com/blockly/reference/js/Blockly.WorkspaceSvg#cleanUp

var strText = 
'<xml xmlns="http://www.w3.org/1999/xhtml">'
+'\n  <variables>'
+'\n    <variable type="">j</variable>'
+'\n    <variable type="">lacarte</variable>'
+'\n    <variable type="">c1</variable>'
+'\n    <variable type="">c2</variable>'
+'\n  </variables>'
+'\n  <block type="text_comment" x="27" y="34">'
+'\n    <value name="COMMENT">'
+'\n      <shadow type="text">'
+'\n        <field name="TEXT">Ajoute des cartes aux cases vide</field>'
+'\n      </shadow>'
+'\n    </value>'
+'\n    <next>'
+'\n      <block type="controls_if">'
+'\n        <value name="IF0">'
+'\n          <block type="logic_compare">'
+'\n            <field name="OP">EQ</field>'
+'\n            <value name="A">'
+'\n              <block type="card_get_card">'
+'\n                <value name="FRAME1">'
+'\n                  <shadow type="math_number">'
+'\n                    <field name="NUM">1</field>'
+'\n                  </shadow>'
+'\n                </value>'
+'\n              </block>'
+'\n            </value>'
+'\n            <value name="B">'
+'\n              <block type="math_number">'
+'\n                <field name="NUM">0</field>'
+'\n              </block>'
+'\n            </value>'
+'\n          </block>'
+'\n        </value>'
+'\n        <statement name="DO0">'
+'\n          <block type="card_add_one_card">'
+'\n            <data>1</data>'
+'\n            <value name="CARD">'
+'\n              <shadow type="math_number">'
+'\n                <field name="NUM">101</field>'
+'\n              </shadow>'
+'\n            </value>'
+'\n            <value name="BACK">'
+'\n              <shadow type="math_number">'
+'\n                <field name="NUM">900</field>'
+'\n              </shadow>'
+'\n            </value>'
+'\n            <value name="FRAME2">'
+'\n              <shadow type="math_number">'
+'\n                <field name="NUM">1</field>'
+'\n              </shadow>'
+'\n            </value>'
+'\n          </block>'
+'\n        </statement>'
+'\n        <next>'
+'\n          <block type="controls_for">'
+'\n            <field name="VAR" variabletype="">j</field>'
+'\n            <value name="FROM">'
+'\n              <shadow type="math_number">'
+'\n                <field name="NUM">2</field>'
+'\n              </shadow>'
+'\n            </value>'
+'\n            <value name="TO">'
+'\n              <shadow type="math_number">'
+'\n                <field name="NUM">300</field>'
+'\n              </shadow>'
+'\n            </value>'
+'\n            <value name="BY">'
+'\n              <shadow type="math_number">'
+'\n                <field name="NUM">1</field>'
+'\n              </shadow>'
+'\n            </value>'
+'\n            <statement name="DO">'
+'\n              <block type="controls_if">'
+'\n                <value name="IF0">'
+'\n                  <block type="logic_compare">'
+'\n                    <field name="OP">EQ</field>'
+'\n                    <value name="A">'
+'\n                      <block type="card_get_card">'
+'\n                        <value name="FRAME1">'
+'\n                          <shadow type="math_number">'
+'\n                            <field name="NUM">1</field>'
+'\n                          </shadow>'
+'\n                          <block type="variables_get">'
+'\n                            <field name="VAR" variabletype="">j</field>'
+'\n                          </block>'
+'\n                        </value>'
+'\n                      </block>'
+'\n                    </value>'
+'\n                    <value name="B">'
+'\n                      <block type="math_number">'
+'\n                        <field name="NUM">0</field>'
+'\n                      </block>'
+'\n                    </value>'
+'\n                  </block>'
+'\n                </value>'
+'\n                <statement name="DO0">'
+'\n                  <block type="controls_if">'
+'\n                    <mutation else="1"></mutation>'
+'\n                    <value name="IF0">'
+'\n                      <block type="logic_compare">'
+'\n                        <field name="OP">LTE</field>'
+'\n                        <value name="A">'
+'\n                          <block type="variables_get">'
+'\n                            <field name="VAR" variabletype="">j</field>'
+'\n                          </block>'
+'\n                        </value>'
+'\n                        <value name="B">'
+'\n                          <block type="math_number">'
+'\n                            <field name="NUM">150</field>'
+'\n                          </block>'
+'\n                        </value>'
+'\n                      </block>'
+'\n                    </value>'
+'\n                    <statement name="DO0">'
+'\n                      <block type="card_add_one_card">'
+'\n                        <data>1</data>'
+'\n                        <value name="CARD">'
+'\n                          <shadow type="math_number">'
+'\n                            <field name="NUM">101</field>'
+'\n                          </shadow>'
+'\n                          <block type="math_random_int">'
+'\n                            <value name="FROM">'
+'\n                              <shadow type="math_number">'
+'\n                                <field name="NUM">102</field>'
+'\n                              </shadow>'
+'\n                            </value>'
+'\n                            <value name="TO">'
+'\n                              <shadow type="math_number">'
+'\n                                <field name="NUM">113</field>'
+'\n                              </shadow>'
+'\n                            </value>'
+'\n                          </block>'
+'\n                        </value>'
+'\n                        <value name="BACK">'
+'\n                          <shadow type="math_number">'
+'\n                            <field name="NUM">900</field>'
+'\n                          </shadow>'
+'\n                        </value>'
+'\n                        <value name="FRAME2">'
+'\n                          <shadow type="math_number">'
+'\n                            <field name="NUM">1</field>'
+'\n                          </shadow>'
+'\n                          <block type="variables_get">'
+'\n                            <field name="VAR" variabletype="">j</field>'
+'\n                          </block>'
+'\n                        </value>'
+'\n                      </block>'
+'\n                    </statement>'
+'\n                    <statement name="ELSE">'
+'\n                      <block type="card_add_one_card">'
+'\n                        <data>1</data>'
+'\n                        <value name="CARD">'
+'\n                          <shadow type="math_number">'
+'\n                            <field name="NUM">101</field>'
+'\n                          </shadow>'
+'\n                          <block type="math_random_int">'
+'\n                            <value name="FROM">'
+'\n                              <shadow type="math_number">'
+'\n                                <field name="NUM">300</field>'
+'\n                              </shadow>'
+'\n                            </value>'
+'\n                            <value name="TO">'
+'\n                              <shadow type="math_number">'
+'\n                                <field name="NUM">313</field>'
+'\n                              </shadow>'
+'\n                            </value>'
+'\n                          </block>'
+'\n                        </value>'
+'\n                        <value name="BACK">'
+'\n                          <shadow type="math_number">'
+'\n                            <field name="NUM">900</field>'
+'\n                          </shadow>'
+'\n                        </value>'
+'\n                        <value name="FRAME2">'
+'\n                          <shadow type="math_number">'
+'\n                            <field name="NUM">1</field>'
+'\n                          </shadow>'
+'\n                          <block type="variables_get">'
+'\n                            <field name="VAR" variabletype="">j</field>'
+'\n                          </block>'
+'\n                        </value>'
+'\n                      </block>'
+'\n                    </statement>'
+'\n                  </block>'
+'\n                </statement>'
+'\n              </block>'
+'\n            </statement>'
+'\n            <next>'
+'\n              <block type="text_comment">'
+'\n                <value name="COMMENT">'
+'\n                  <shadow type="text">'
+"\n                    <field name='TEXT'>Cliquez sur l'As de pique pour arrêter</field>"
+'\n                  </shadow>'
+'\n                </value>'
+'\n                <next>'
+'\n                  <block type="card_reset_click_count">'
+'\n                    <value name="FRAME1">'
+'\n                      <shadow type="math_number">'
+'\n                        <field name="NUM">1</field>'
+'\n                      </shadow>'
+'\n                    </value>'
+'\n                    <next>'
+'\n                      <block type="variables_set">'
+'\n                        <field name="VAR" variabletype="">lacarte</field>'
+'\n                        <value name="VALUE">'
+'\n                          <block type="math_number">'
+'\n                            <field name="NUM">1</field>'
+'\n                          </block>'
+'\n                        </value>'
+'\n                        <next>'
+'\n                          <block type="controls_whileUntil">'
+'\n                            <field name="MODE">WHILE</field>'
+'\n                            <value name="BOOL">'
+'\n                              <block type="logic_compare">'
+'\n                                <field name="OP">LT</field>'
+'\n                                <value name="A">'
+'\n                                  <block type="card_get_card_click_count">'
+'\n                                    <value name="FRAME1">'
+'\n                                      <shadow type="math_number">'
+'\n                                        <field name="NUM">1</field>'
+'\n                                      </shadow>'
+'\n                                      <block type="variables_get">'
+'\n                                        <field name="VAR" variabletype="">lacarte</field>'
+'\n                                      </block>'
+'\n                                    </value>'
+'\n                                  </block>'
+'\n                                </value>'
+'\n                                <value name="B">'
+'\n                                  <block type="math_number">'
+'\n                                    <field name="NUM">1</field>'
+'\n                                  </block>'
+'\n                                </value>'
+'\n                              </block>'
+'\n                            </value>'
+'\n                            <statement name="DO">'
+'\n                              <block type="text_pause">'
+'\n                                <field name="SPEED">NONE</field>'
+'\n                                <value name="TIME">'
+'\n                                  <shadow type="math_number">'
+'\n                                    <field name="NUM">0.01</field>'
+'\n                                  </shadow>'
+'\n                                </value>'
+'\n                                <next>'
+'\n                                  <block type="variables_set">'
+'\n                                    <field name="VAR" variabletype="">c1</field>'
+'\n                                    <value name="VALUE">'
+'\n                                      <block type="math_random_int">'
+'\n                                        <value name="FROM">'
+'\n                                          <shadow type="math_number">'
+'\n                                            <field name="NUM">1</field>'
+'\n                                          </shadow>'
+'\n                                        </value>'
+'\n                                        <value name="TO">'
+'\n                                          <shadow type="math_number">'
+'\n                                            <field name="NUM">300</field>'
+'\n                                          </shadow>'
+'\n                                        </value>'
+'\n                                      </block>'
+'\n                                    </value>'
+'\n                                    <next>'
+'\n                                      <block type="variables_set">'
+'\n                                        <field name="VAR" variabletype="">c2</field>'
+'\n                                        <value name="VALUE">'
+'\n                                          <block type="math_random_int">'
+'\n                                            <value name="FROM">'
+'\n                                              <shadow type="math_number">'
+'\n                                                <field name="NUM">1</field>'
+'\n                                              </shadow>'
+'\n                                            </value>'
+'\n                                            <value name="TO">'
+'\n                                              <shadow type="math_number">'
+'\n                                                <field name="NUM">300</field>'
+'\n                                              </shadow>'
+'\n                                            </value>'
+'\n                                          </block>'
+'\n                                        </value>'
+'\n                                        <next>'
+'\n                                          <block type="card_exchangef1f2">'
+'\n                                            <data>1</data>'
+'\n                                            <value name="FRAME1">'
+'\n                                              <shadow type="math_number">'
+'\n                                                <field name="NUM">3</field>'
+'\n                                              </shadow>'
+'\n                                              <block type="variables_get">'
+'\n                                                <field name="VAR" variabletype="">c1</field>'
+'\n                                              </block>'
+'\n                                            </value>'
+'\n                                            <value name="FRAME2">'
+'\n                                              <shadow type="math_number">'
+'\n                                                <field name="NUM">5</field>'
+'\n                                              </shadow>'
+'\n                                              <block type="variables_get">'
+'\n                                                <field name="VAR" variabletype="">c2</field>'
+'\n                                              </block>'
+'\n                                            </value>'
+'\n                                            <next>'
+'\n                                              <block type="text_comment">'
+'\n                                                <value name="COMMENT">'
+'\n                                                  <shadow type="text">'
+"\n                                                    <field name='TEXT'>Pour suivre la case de l'As de pique</field>"
+'\n                                                  </shadow>'
+'\n                                                </value>'
+'\n                                                <next>'
+'\n                                                  <block type="controls_if">'
+'\n                                                    <mutation else="1"></mutation>'
+'\n                                                    <value name="IF0">'
+'\n                                                      <block type="logic_compare">'
+'\n                                                        <field name="OP">EQ</field>'
+'\n                                                        <value name="A">'
+'\n                                                          <block type="variables_get">'
+'\n                                                            <field name="VAR" variabletype="">lacarte</field>'
+'\n                                                          </block>'
+'\n                                                        </value>'
+'\n                                                        <value name="B">'
+'\n                                                          <block type="variables_get">'
+'\n                                                            <field name="VAR" variabletype="">c1</field>'
+'\n                                                          </block>'
+'\n                                                        </value>'
+'\n                                                      </block>'
+'\n                                                    </value>'
+'\n                                                    <statement name="DO0">'
+'\n                                                      <block type="variables_set">'
+'\n                                                        <field name="VAR" variabletype="">lacarte</field>'
+'\n                                                        <value name="VALUE">'
+'\n                                                          <block type="variables_get">'
+'\n                                                            <field name="VAR" variabletype="">c2</field>'
+'\n                                                          </block>'
+'\n                                                        </value>'
+'\n                                                      </block>'
+'\n                                                    </statement>'
+'\n                                                    <statement name="ELSE">'
+'\n                                                      <block type="controls_if">'
+'\n                                                        <value name="IF0">'
+'\n                                                          <block type="logic_compare">'
+'\n                                                            <field name="OP">EQ</field>'
+'\n                                                            <value name="A">'
+'\n                                                              <block type="variables_get">'
+'\n                                                                <field name="VAR" variabletype="">lacarte</field>'
+'\n                                                              </block>'
+'\n                                                            </value>'
+'\n                                                            <value name="B">'
+'\n                                                              <block type="variables_get">'
+'\n                                                                <field name="VAR" variabletype="">c2</field>'
+'\n                                                              </block>'
+'\n                                                            </value>'
+'\n                                                          </block>'
+'\n                                                        </value>'
+'\n                                                        <statement name="DO0">'
+'\n                                                          <block type="variables_set">'
+'\n                                                            <field name="VAR" variabletype="">lacarte</field>'
+'\n                                                            <value name="VALUE">'
+'\n                                                              <block type="variables_get">'
+'\n                                                                <field name="VAR" variabletype="">c1</field>'
+'\n                                                              </block>'
+'\n                                                            </value>'
+'\n                                                          </block>'
+'\n                                                        </statement>'
+'\n                                                      </block>'
+'\n                                                    </statement>'
+'\n                                                  </block>'
+'\n                                                </next>'
+'\n                                              </block>'
+'\n                                            </next>'
+'\n                                          </block>'
+'\n                                        </next>'
+'\n                                      </block>'
+'\n                                    </next>'
+'\n                                  </block>'
+'\n                                </next>'
+'\n                              </block>'
+'\n                            </statement>'
+'\n                            <next>'
+'\n                              <block type="text_afficheln">'
+'\n                                <value name="TEXT">'
+'\n                                  <shadow type="text">'
+'\n                                    <field name="TEXT">Terminé</field>'
+'\n                                  </shadow>'
+'\n                                </value>'
+'\n                                <next>'
+'\n                                  <block type="card_exchangef1f2">'
+'\n                                    <data>1</data>'
+'\n                                    <value name="FRAME1">'
+'\n                                      <shadow type="math_number">'
+'\n                                        <field name="NUM">3</field>'
+'\n                                      </shadow>'
+'\n                                      <block type="variables_get">'
+'\n                                        <field name="VAR" variabletype="">lacarte</field>'
+'\n                                      </block>'
+'\n                                    </value>'
+'\n                                    <value name="FRAME2">'
+'\n                                      <shadow type="math_number">'
+'\n                                        <field name="NUM">1</field>'
+'\n                                      </shadow>'
+'\n                                    </value>'
+'\n                                  </block>'
+'\n                                </next>'
+'\n                              </block>'
+'\n                            </next>'
+'\n                          </block>'
+'\n                        </next>'
+'\n                      </block>'
+'\n                    </next>'
+'\n                  </block>'
+'\n                </next>'
+'\n              </block>'
+'\n            </next>'
+'\n          </block>'
+'\n        </next>'
+'\n      </block>'
+'\n    </next>'
+'\n  </block>'
+'\n</xml>'
+'\n';

var xml = Blockly.Xml.textToDom(strText);
Blockly.Xml.domToWorkspace(xml, gloB.demoWorkspace);
} // confStartBlocksInit

function confParamInit() {
//========================
// Définition de certains paramètres
// c.f. bglibrary_frame_card.js
// c.f. bglibrary_abase.js
gloCanvas.marginRight = 5;  // marge entre la droite du dernier cadre et la droite du canvas
gloCanvas.marginBottom = 5 + 6; // marge entre le bas du dernier cadre et le bas du canvas   + 6 pour le numéro de la case.
gloCanvas.backgroundColor = "rgb(255,255,250)"; // Couleur de fond du canvas

gloCanvas.cHide = ""; // Si vide, accepte la superposition de cartes.
                      // Si == "c", une carte placée sur une autre fait disparaitre la carte du dessous.
                      //            donc n'accepte pas la superposition de cartes.

// Pour les pas de déplacement de la carte sur sa trajectoire        
gloCanvas.nStep = 0;
gloCanvas.nStepStop = 0;

gloCanvas.nCardsShift = 10;  // Pour décaler les cartes se trouvant sur le même tas.

// Sont défini dans : confFrameCardsInit
gloCanvas.strSource = "../images/Carte"; // Source des images des cartes, un nombre entre 100 et 999 suivra
gloCanvas.strExt = ".png"; // Extension des images des cartes. ".png", ".jpg" et ".gif" est habituel.

// Si on enlève les commentaires.
// Change le fond de la fenêtre d'affichage ou de code Javascript ou Python ou Lua ou d'affichage des variables
//document.getElementById('idDisplayTextArea').style.backgroundColor = 'rgb(200, 255, 255)';
//document.getElementById('idCodeJavascriptTextArea').style.backgroundColor = 'rgb(200, 255, 255)';
//document.getElementById('idCodePythonTextArea').style.backgroundColor = 'rgb(200, 255, 255)';
//document.getElementById('idCodeLuaTextArea').style.backgroundColor = 'rgb(200, 255, 255)';
//document.getElementById('idVariableSpyTextArea').style.backgroundColor = 'rgb(200, 255, 255)';
// Ne pas changer ici. document.getElementById('idFrameCardsCanvas').style.backgroundColor = 'rgb(200, 255, 255)';  // c.f. gloCanvas.backgroundColor

// Exemple de changement de position et dimension de fenêtres
// function window_config(idWin, nVisible, nPosX, nPosY, nWidth, nHeight)  définit dans : ex2000.js
// idWin est une référence à la fenêtre à modifier.
// nVisible == 1 pour être visible ; == 0 pour être caché ; autre => ne change pas la visibilité 
// Positions et dimensions en pixels.  -9999 => reste inchangé.
// dimension <= 0 => reste inchangé
window_config('idDisplay', 1, -9999, -9999, 0, 0); // Fenêtre d'affichage, visible
window_config('idCodeBlockly', 1, -9999, -9999, 0, 0); // Fenêtre Blockly, visible
window_config('idCodeJavascript', -1, -9999, -9999, 0, 0); // Fenêtre Javascript, reste inchangée
window_config('idCodePython', -1, -9999, -9999, 0, 0); // Fenêtre Python, reste inchangée
window_config('idCodeLua', -1, -9999, -9999, 0, 0); // Fenêtre Lua, fenêtre cachée, reste inchangée
window_config('idVariableSpy', -1, -9999, -9999, 0, 0); // Fenêtre des valeurs de variables, reste inchangée
window_config('idFrameCards', 1, -9999, -9999, 0, 0); // Fenêtre du jeu de cartes, visible

} // confParamInit

function confFramesInit() {
//=========================
// Initialisation des cadres
 
// Définition de quelques cadres
// Les unités de position et de tailles sont arbitraires, car
// elles s'adaptent aux dimensions du canvas.
var nXmax = 20; // nombre de colonnes
var nYmax = 15; // nombre de lignes
var nn = 0;
for (var nY=0; nY<nYmax; nY++) {
  for (var nX=0; nX<nXmax; nX++) {
    nn++;
    glaoFrames[nn].posX = 5 + 96*nX;
    glaoFrames[nn].posY = 5 + (144 + 5)*nY;  // + 5 pour le numéro de la case.
    glaoFrames[nn].deltaX = 86;
    glaoFrames[nn].deltaY = 134;
    }
  }
glnFramesMax = nn;

// Le cadre d'indice 0 sert à cacher des cartes ou
// est le point de départ d'apparition de nouvelles cartes.
// Ce cadre est défini dans la fonction myFramesInit() du fichier ex2000.js
} // confFramesInit

function confCardsInit() {
//========================
// Placement de cartes
// Aucune carte au départ.
} // confCardsInit

function confMessageBegin() {
//==========================
// Permet d'afficher un message au départ,
// que l'on peut revoir en cliquant sur le point d'interrogation
// Si on en veut aucun, on peut laisser cette fonction vide, 
// ou ne pas l'appeler ci-dessous.

// &nbsp;  insert un espace
// <br>    insert un retour à la ligne 
// <b> .. </b>  pour mettre du texte en gras
// toutes les balises HTML sont acceptée, y compris les styles CSS.

// Consultez le code source du fichier "ex2000.html" pour d'autres "id" 
// permettant de faire d'autres modifications depuis ici.

// Change le contenu de la fenêtre "A'de..."
document.getElementById('HTML_MAIN_TITLE2').innerHTML =
  "&nbsp; Exemple de <b>gestion d'événement</b>.<br>"
+ "<br>"  // <br> est une balise pour sauter une ligne.  Toutes la synthaxe HTML est disponible. 
+ "&nbsp; Exécutez à vitesse maximale (>>>). Patientez quelques secondes. &nbsp;<br>"
+ "&nbsp; Les cartes bouges, tant que l'on a pas cliqué sur l'As. &nbsp;<br>"
+ "<br>"  // <br> est une balise pour sauter une ligne.  Toutes la synthaxe HTML est disponible. 
+ "&nbsp; Tentez des modifications du code."
;

// Changement du titre du message
document.getElementById('HTML_MESSAGE').innerHTML =
  "&nbsp; Message à propos de cette configuration. &nbsp;"
;

// Change le contenu de la fenêtre "À propos..."
document.getElementById('HTML_ABOUT2').innerHTML =
  "&nbsp; Configuration créée par Bernard Gisin &nbsp;<br>"
+ "&nbsp; le 4 janvier 2020."
+ "<br><br>"
;

// Changement du titre de la boîte de dialogue "À propos de..."
document.getElementById('HTML_ABOUT1').innerHTML =
  "&nbsp; À propos de cette configuration. &nbsp;"
;

DialogHelp(true);
} // confMessageBegin

// Exécution des fonctions d'initialisations.
ConfigInitClear(); // Pour préparer une nouvelle configuration
confToolBoxInit();
confStartBlocksInit();
confParamInit();
confFramesInit();
confCardsInit();

// Pour placer les cartes correctement dans les cadres.
myFrameCardsResize(null);

// Message juste après le chargement de la configuration.
confMessageBegin();
